/*
 * Quoridor server and related libraries
 * Copyright (C) 2009-2010 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://www.gnu.org/licenses/.
 */
package cz.xelfi.quoridor;

/**
 * Represents position of a fence on a {@link Board}. The fence can be oriented 
 * either in
 * {@link Orientation#HORIZONTAL} or {@link Orientation#VERTICAL} direction.
 * The faces postions are numbered 1-8 and A-H according to following
 * graph: <pre>
            H   G   F   E   D   C   B   A
            |   |   |   |   |   |   |   |
        +-----------------------------------+
        |                                   |
     1--|   +   +   +   +   +   +   +   +   |--1
        |                                   |
     2--|   +   +   +   +   +   +   +   +   |--2
        |                                   |
     3--|   +   +   +   +   +   +   +   +   |--3
        |                                   |
     4--|   +   +   +   +   +   +   +   +   |--4
[E]     |                                   |     [W]
     5--|   +   +   +   +   +   +   +   +   |--5
        |                                   |
     6--|   +   +   +   +   +   +   +   +   |--6
        |                                   |
     7--|   +   +   +   +   +   +   +   +   |--7
        |                                   |
     8--|   +   +   +   +   +   +   +   +   |--8
        |                                   |
        +-----------------------------------+
            |   |   |   |   |   |   |   |
            H   G   F   E   D   C   B   A
     * </pre>
 *
 */
public final class Fence extends Object {

    private final int x;
    private final int y;
    private final Orientation o;

    Fence(int x, int y, Orientation o) {
        super();
        this.x = x;
        this.y = y;
        this.o = o;
    }

    /** The column of the center of the fence. The same
     * as used in {@link Move#fence(char, int, cz.xelfi.quoridor.Fence.Orientation)}.
     *
     * @return a letter between 'A' and 'H'
     */
    public char getColumn() {
        return (char) ('A' + (x - 1) / 2);
    }

    /** The row of the center of the fence. The same
     * as specified in {@link Move#fence(char, int, cz.xelfi.quoridor.Fence.Orientation)}.
     *
     * @return a number between 1 and 8
     */
    public int getRow() {
        return (y + 1) / 2;
    }

    /** Midle coordinate of the fence.
     * @return 1-15
     */
    int getX() {
        return x;
    }

    /** Midle coordinate of the fence.
     * @return 1-15
     */
    int getY() {
        return y;
    }

    /** The orientation of the fence.
     * 
     * @return HORIZONTAL or VERTICAL
     */
    public Orientation getOrientation() {
        return o;
    }

    @Override
    public String toString() {
        return "Fence[" + getColumn() + "," + getRow() + "," + o + "]";
    }

    @Override
    public int hashCode() {
        return 8 * x + 4 * y + 13 + o.hashCode();
    }

    @Override
    public boolean equals(Object o) {
        if (o instanceof Fence) {
            Fence f = (Fence) o;
            return x == f.x && y == f.y && getOrientation().equals(f.getOrientation());
        }
        return false;
    }

    /** The possible orientation of a {@link Fence}.
     * Either horizontal of vertical.
     */
    public static enum Orientation {
        HORIZONTAL, VERTICAL;

        private Orientation() {
        }
    }
}
